#*-* coding: utf-8 *-*
import json
import os
from McuCommunicator import *
from AntennaSubCommunicator import *
from Antenna import *
from AntCommunicator import *
from WebCommunicator import *
from System import *
from VCS import *
from NmeaCommunicator import *
from ConsoleCommunicator import *
from GyroCommunicator import *
from GpsManager import *
from GyroManager import *
from BlockControl import *
from AbsCommunicator import *
from AbsServer import *
from Log import *
from Util import *
from threading import Thread
from multiprocessing import Process


class Project:
    def __init__(self):
        self.Version = '0.346'
        self.AntAutoSel = 0     #Selected antenna index for auto mode
        self.AntAInterface= 0         #Selected antenna interface of A
        self.AntBInterface = 0        #Selected antenna interface of B
        self.AntA = None        
        self.AntB = None
        self.SelectedAnt = None
        self.RefFreq = 0
        self.AntATrackOn = 0
        self.AntBTrackOn = 0
        self.AntIfSel = 0
        self.GyroSwitch = 0
        self.GyroReady = False
        self.UpdateSysTimeCnt = 0;
        self.InitializeReady = False
        self.ResetFlag=False
        #Gyro - sync&step exception
        self.GyroSyncStepStartupFlag = True
        self.LastAntSelect = 0
        
        #Log
        self.Log = Log("Log") 
        self.Log.Load()
        self.Log.PushBootEvent()
        
        # ------------ Initialize System Step.0, Configuration ------------
        self.Sys = System("System") 
        self.Sys.Load()
        
        
        # ------------ Initialize System Step.1, MCU <-> RASP   ------------
		# * VCS
        self.Vcs = VCS()
		# * McuCommunicator
        self.McuComm = McuCommmunicator("McuComm") 
        self.McuComm.Open(self.Sys.GetMcuName(), self.Sys.GetMcuBaudrate())
        p1 = Thread(target=self.McuComm.Start) 
        p1.start()
		# * Init BD-Mux Setting
        self.Vcs.AntSel = self.Sys.Dat['Vcs']['Default']['AntSel']
        self.Vcs.RefFreqSel = self.Sys.Dat['Vcs']['Default']['RefFreq']
        self.McuComm.Mcu.AttnRx1 = self.Sys.Dat['Vcs']['Default']['AttnRx1']
        self.McuComm.Mcu.AttnRx2 = self.Sys.Dat['Vcs']['Default']['AttnRx2']
        self.McuComm.Mcu.AttnTx1 = self.Sys.Dat['Vcs']['Default']['AttnTx1']
        self.McuComm.Mcu.AttnTx2 = self.Sys.Dat['Vcs']['Default']['AttnTx2']
        self.McuComm.SendAntSelect(self.Vcs.AntSel)
        self.McuComm.SendRefFreqSelect(self.Vcs.RefFreqSel)
        self.McuComm.SendAttn(0,self.McuComm.Mcu.AttnRx1)
        self.McuComm.SendAttn(1,self.McuComm.Mcu.AttnRx2)
        self.McuComm.SendAttn(2,self.McuComm.Mcu.AttnTx1)
        self.McuComm.SendAttn(3,self.McuComm.Mcu.AttnTx2)
        self.InitializeReady = True
        
        
        # ------------ Initialize System Step.2, Interfaces ------------
        self.AntCommA = AntCommunicator("AntCommA")
        self.AntCommA.Init(self.Sys.GetAntAIP(), self.Sys.GetAntAPort())
        p2 = Thread(target=self.AntCommA.Start) 
        p2.start()
        self.AntCommB = AntCommunicator("AntCommB")
        self.AntCommB.Init(self.Sys.GetAntBIP(), self.Sys.GetAntBPort()) 
        p3 = Thread(target=self.AntCommB.Start) 
        p3.start()
        self.AntSubComm = AntennaSubCommunicator("AntSubComm")
        self.AntSubComm.Open(self.Sys.GetAntSubName(), self.Sys.GetAntSubBaudrate())
        p4 = Thread(target=self.AntSubComm.Start) 
        p4.start()
        self.WebComm = WebCommunicator('WebComm')
        self.WebComm.Init(self.Sys.GetWebIP(), self.Sys.GetWebPort())
        p5 = Thread(target=self.WebComm.Start) 
        p5.start() 
        self.NmeaComm = NmeaCommunicator("NmeaComm") 
        self.NmeaComm.Open(self.Sys.GetNmeaName(), self.Sys.GetNmeaBaudrate())
        p6 = Thread(target=self.NmeaComm.Start) 
        p6.start()
        self.ConComm = ConsoleCommunicator("ConComm") 
        self.ConComm.Open(self.Sys.GetConsoleName(), self.Sys.GetConsoleBaudrate())
        p7 = Thread(target=self.ConComm.Start) 
        p7.start()
        self.GyroComm = GyroCommunicator("GyroComm") 
        self.GyroComm.Open(self.Sys.GetGyroName(), self.Sys.GetGyroBaudrate())
        p8 = Thread(target=self.GyroComm.Start) 
        p8.start()
        self.AbsSer = AbsServer("AbsSer") 
        self.AbsSer.Init('', self.Sys.GetAbsPort())
        p9 = Thread(target=self.AbsSer.Start) 
        p9.start()
        # test
        self.AbsComm = AbsCommunicator("AbsComm")
        self.AbsComm.Init('127.0.0.1', 52012)
        p10 = Thread(target=self.AbsComm.Start) 
        p10.start()
        # ------------ Initialize System Step.3 Management      ------------
        self.GpsMgr = GpsManager()
        self.GyroMgr = GyroManager()
        self.BlockCtl = BlockControl()
        # ------------ Initialize System Step.4 Inner Workers   ------------
        # Workers
        w1 = Thread(target=self.Worker)
        w1.start()
        w2 = Thread(target=self.SubWorker)
        w2.start()
        w3 = Thread(target=self.ResetWorker)
        w3.start()  
    def Reboot(self): self.ResetFlag = True
    def ResetWorker(self):
        time.sleep(3)
        ResetCnt = 0
        while True:
            time.sleep(0.1)
            if self.ResetFlag : ResetCnt += 1
            if ResetCnt == 20 : os.system('sudo reboot -f') 
    def SubWorker(self):
        time.sleep(3)
        lock = False
        while True:
            time.sleep(1)
            lock = self.ConComm.getCD()
            lock = not lock if self.Sys.GetAbsConsolOutputType() == 1 else lock
            self.Vcs.LockFromConsole = 1 if lock else 0
    def Worker(self):
        # Set current NMEA mode.
        if self.Sys.GetGyroType() != 2 : self.GyroSyncStepStartupFlag = False
        
        time.sleep(2)
        self.McuComm.Send(Util.AppendCRC('$KB,SYS,100,0,00,0,0,'))
        pj.GyroComm.SendForCustom(int(self.Sys.Dat['Gyro']['Direction'])
                                ,int(self.Sys.Dat['Gyro']['SyncFreq'])
                                ,int(self.Sys.Dat['Gyro']['ManualType'])
                                ,float(self.Sys.Dat['Gyro']['Converter']))
        time.sleep(1)
        self.GyroReady = True
        # Check list
        chk_gps = 0
        chk_gyro = 0
        chk_link = 0
        chk_lock = 0
        chk_antsw_in_auto = 0
        chk_dec = 0
        
        # *Gyro - sync&step exception
        # case. self.Sys.GetGyroType() == 2
        
        reflesh_cnt = 0
        io_gyro = (1 if self.Sys.GetGyroType() == 2 else 0)
        io_block = (1 if self.BlockCtl.IsBlock == False else 0)
        io_track = self.BlockCtl.Track
        led_gyro = (3 if self.GyroSyncStepStartupFlag else (1 if self.GyroMgr.Valid > 0 else 0))
        led_anta = self.AntATrackOn
        led_antb = self.AntBTrackOn
        led_link = self.Vcs.Link
        led_lock = self.Vcs.Lock
        led_auto = (1 if self.Vcs.AntSel == 3 else 0)
        led_autosel = self.AntAutoSel
                        
        old_io_gyro =   io_gyro  = 0
        old_io_block =  io_block = 0
        old_io_track =  io_track = 0
        old_led_gyro =  led_gyro = 0
        old_led_anta =  led_anta = 0
        old_led_antb =  led_antb = 0
        old_led_link =  led_link = 0
        old_led_lock =  led_lock = 0
        old_led_auto =  led_auto = 0
        old_led_autosel = led_autosel = 0
        
        
        while True:
			time.sleep(0.2)
            
			# Sys Reset
			if self.McuComm.Mcu.SysReset == 1 :
				dat1 = '$KB,SYS,%d%d%d0,%d,%d%d,%d,%d,%d,%d,'%(
                                        self.GyroSwitch, (1 if self.BlockCtl.IsBlock == False else 0), self.BlockCtl.Track, (3 if self.GyroSyncStepStartupFlag else (1 if self.GyroMgr.Valid > 0 else 0)), 
                                        self.AntATrackOn,self.AntBTrackOn,self.Vcs.Link,
                                        0,(1 if self.Vcs.AntSel == 3 else 0),self.AntAutoSel)
				dat2 = '$KB,SYS,%d%d%d0,%d,%d%d,%d,%d,%d,%d,'%(
                                        self.GyroSwitch, (1 if self.BlockCtl.IsBlock == False else 0), self.BlockCtl.Track, (3 if self.GyroSyncStepStartupFlag else (1 if self.GyroMgr.Valid > 0 else 0)), 
                                        self.AntATrackOn,self.AntBTrackOn,self.Vcs.Link,
                                        1,(1 if self.Vcs.AntSel == 3 else 0),self.AntAutoSel)
				self.McuComm.Send(Util.AppendCRC(dat1))
				time.sleep(0.1)
				self.McuComm.Send(Util.AppendCRC(dat2))
				time.sleep(0.1)
				self.McuComm.Send(Util.AppendCRC(dat1))
				time.sleep(0.1)
				self.McuComm.Send(Util.AppendCRC(dat2))
				time.sleep(0.1)
				self.McuComm.Send(Util.AppendCRC(dat1))
				time.sleep(0.1)
				self.McuComm.Send(Util.AppendCRC(dat2))
				time.sleep(0.1)
				self.McuComm.Send(Util.AppendCRC(dat1))
				time.sleep(0.1)
				self.McuComm.Send(Util.AppendCRC(dat2))
				time.sleep(0.1)
				self.McuComm.Send(Util.AppendCRC(dat1))
				time.sleep(0.1)
				self.McuComm.Send(Util.AppendCRC(dat2))
				time.sleep(0.1)
				self.McuComm.Send(Util.AppendCRC(dat1))
				time.sleep(0.1)
				self.McuComm.Send(Util.AppendCRC(dat2))
				time.sleep(0.1)
				self.McuComm.Send(Util.AppendCRC(dat1))
				time.sleep(0.1)
				self.McuComm.Send(Util.AppendCRC(dat2))
				time.sleep(0.1)
				Util.SetDefaultIPv4() 
				self.McuComm.Mcu.SysReset = 0;
				self.Reboot()
            
            
            # 1. Check antenna interface both tcp and serial
			self.SetValidInterfaceForAntenna()
            # 2. Check link of antenna and select antenna in automode.
			self.UpdateAntLink()
            # 3. Check Ant Tracking Flag
			self.AntATrackOn = self.GetStatusAntenna(self.AntA, self.AntAInterface) 
			self.AntBTrackOn = self.GetStatusAntenna(self.AntB, self.AntBInterface)
            # 4. Check Ant Tracking Flag
			self.AntAutoSel = self.UpdateAntAutoSel(self.AntATrackOn, self.AntBTrackOn)
            
			io_gyro = (1 if self.Sys.GetGyroType() == 2 else 0)
			io_block = (1 if self.BlockCtl.IsBlock == False else 0)
			io_track = self.BlockCtl.Track
			led_gyro = (3 if self.GyroSyncStepStartupFlag else (1 if self.GyroMgr.Valid > 0 else 0))
			led_anta = self.AntATrackOn
			led_antb = self.AntBTrackOn
			led_link = self.Vcs.Link
			led_lock = self.Vcs.Lock
			led_auto = (1 if self.Vcs.AntSel == 3 else 0)
			led_autosel = self.AntAutoSel
            
			if reflesh_cnt < 5 and (old_io_gyro == io_gyro and old_io_block == io_block and old_io_track == io_track and old_led_gyro == led_gyro and old_led_anta == led_anta and old_led_antb == led_antb  and old_led_link == led_link  and old_led_lock == led_lock  and old_led_auto == led_auto  and old_led_autosel == led_autosel) :
				reflesh_cnt = reflesh_cnt + 1
				continue
                
			old_io_gyro =   io_gyro 
			old_io_block =  io_block
			old_io_track =  io_track 
			old_led_gyro =  led_gyro 
			old_led_anta =  led_anta 
			old_led_antb =  led_antb 
			old_led_link =  led_link 
			old_led_lock =  led_lock 
			old_led_auto =  led_auto 
			old_led_autosel = led_autosel 
			reflesh_cnt = 0
            
			
			# Test print
			# dat = '$KB,SYS,%d%d%d,%d,%d%d,%d,%d,'%(0,0,0,0,1,1,1,0)
            # -----------------------------------------
            # self.GyroSwitch == 1, Custom(Sync, Step)
            # self.GyroSwitch == 0, NMEA0183
			self.GyroSwitch = io_gyro
			dat = '$KB,SYS,%d%d%d0,%d,%d%d,%d,%d,%d,%d,'%(
                                        self.GyroSwitch,
                                        (1 if self.BlockCtl.IsBlock == False else 0),
                                        self.BlockCtl.Track,
                                        (3 if self.GyroSyncStepStartupFlag else (1 if self.GyroMgr.Valid > 0 else 0)), 
                                        # self.GyroMgr.Valid, # defined the manual to valid.
                                        self.AntATrackOn,
                                        self.AntBTrackOn,
                                        self.Vcs.Link,
                                        self.Vcs.Lock,
                                        (1 if self.Vcs.AntSel == 3 else 0),
                                        self.AntAutoSel)
			self.McuComm.Send(Util.AppendCRC(dat))
			# print(Util.AppendCRC(dat))
            
			if chk_gps != self.GpsMgr.Valid : 
				chk_gps = self.GpsMgr.Valid
				pj.Log.PushGpsValid(chk_gps)
			if chk_gyro != self.GyroMgr.Valid : 
				chk_gyro = self.GyroMgr.Valid
				pj.Log.PushGyroValid(chk_gyro)
			if chk_link != self.Vcs.Link : 
				chk_link = self.Vcs.Link
				pj.Log.PushAntLink(chk_link)
			if chk_lock != self.Vcs.Lock : 
				chk_lock = self.Vcs.Lock
				pj.Log.PushAntLock(chk_lock)
			if self.Vcs.AntSel == 3 :
				if chk_antsw_in_auto != self.AntAutoSel : 
					chk_antsw_in_auto = self.AntAutoSel
					pj.Log.PushAntSwitching(chk_antsw_in_auto)
			else : chk_antsw_in_auto = self.AntAutoSel
			if chk_dec != self.McuComm.Mcu.DecTxRefFreq : 
				chk_dec = self.McuComm.Mcu.DecTxRefFreq
				pj.Log.PushRefDec(chk_dec)
			# print dat
			# print "Ant Selection :%d - %d / Active : %d -  %d / Connection:  %d - %d"%(self.AntAInterface ,self.AntBInterface ,self.AntATrackOn ,self.AntBTrackOn ,self.AntCommB.IsConnect() ,self.McuComm.SubAnt.IsConnect())
    def UpdateAntAutoSel(self,a,b):  
        if self.AntAutoSel == 1 and a == 1 : return 1; 
        elif self.AntAutoSel == 2 and b == 1 : return 2; 
        
        reval = 0;
        if a == 1 : reval = 1
        elif b == 1 : reval = 2 
        
        #new - track and block situlation exception
        if a != 1 and b != 1 :
            if self.GetStatusAntennaPreOper(self.AntA, self.AntAInterface)==1 : reval = 1; 
            elif self.GetStatusAntennaPreOper(self.AntB, self.AntBInterface)==1 : reval = 2; 
        
        if self.Vcs.AntSel == 3 and (reval == 1 or reval == 2 or reval == 0) and (self.LastAntSelect != reval) : 
            self.McuComm.SendAntSelect(reval)
            self.LastAntSelect = reval
        # print reval
        return reval
    def UpdateAntLink(self):
        self.SelectedAnt = None 
        if self.Vcs.AntSel == 3 :
            if self.AntAutoSel == 1 : 
				self.SelectedAnt = self.AntA
				self.AntIfSel = self.AntAInterface
            elif self.AntAutoSel == 2 : 
				self.SelectedAnt = self.AntB
				self.AntIfSel = (self.AntBInterface + (2 if self.AntBInterface > 0 else 0))
        elif self.Vcs.AntSel == 1 : 
            self.SelectedAnt = self.AntA
            self.AntIfSel = self.AntAInterface
        elif self.Vcs.AntSel == 2 :
            self.SelectedAnt = self.AntB
            self.AntIfSel = (self.AntBInterface + (2 if self.AntBInterface > 0 else 0))
        # Check link of selected antenna
        if self.SelectedAnt != None : self.Vcs.Link = int(self.SelectedAnt.Status.GetWithToken('NimLock')) 
        else : self.Vcs.Link = 0
    def GetStatusAntenna(self, ant, ant_if):
        reval = 0
        if ant_if > 0 :
			track = int(ant.Status.GetWithToken('TrackingStatus'))
			status = int(ant.Status.GetWithToken('DrivingStatus'))
			driving = int(ant.Status.GetWithToken('Driving'))
			txon = int(ant.Status.GetWithToken('TxMute'))
			asipass = int(ant.Status.GetWithToken('AsiPass'))
			mode = int(ant.Status.GetWithToken('DrivingMode'))
			# print "GetStatusAntenna ("+ant.Name+"): " + str(ant_if) + " / " + str(track) + " / " + str(status) + " / " + str(driving) + " / " + str(txon) + " / " + str(asipass)+ " / " + str(mode)
			if driving==1 :
				if status == 1 : reval = 2 
				elif status > 1 and status < 4 :
					if track==1 and txon==1 and asipass==1 and (mode>1 and mode<4) : reval = 1
					else : reval = 2
				else : reval = 3
        return reval
    def GetStatusAntennaPreOper(self, ant, ant_if):
        reval = 0
        if ant_if > 0 :
			track = int(ant.Status.GetWithToken('TrackingStatus'))
			status = int(ant.Status.GetWithToken('DrivingStatus'))
			driving = int(ant.Status.GetWithToken('Driving'))
			asipass = int(ant.Status.GetWithToken('AsiPass'))
			mode = int(ant.Status.GetWithToken('DrivingMode'))
			# print "GetStatusAntenna ("+ant.Name+"): " + str(ant_if) + " / " + str(track) + " / " + str(status) + " / " + str(driving) + " / " + str(txon) + " / " + str(asipass)+ " / " + str(mode)
			if driving==1 :
				if status == 1 : reval = 2 
				elif status > 1 and status < 4 :
					if track==1 and asipass==1 and (mode>1 and mode<4) : reval = 1
					else : reval = 2
				else : reval = 3
        return reval
    def SetValidInterfaceForAntenna(self):
        anta_inf = antb_inf = 0
        if self.AntCommA.IsConnect() == 1 : anta_inf = 1
        elif self.AntSubComm.IsConnect() == 1 : anta_inf = 2
        if self.AntAInterface != anta_inf :
            self.AntAInterface = anta_inf
            #Set antenna information.
            if anta_inf == 1 : self.AntA = self.AntCommA.Ant
            elif anta_inf == 2 : self.AntA = self.AntSubComm.Ant
            else : self.AntA = None
            #Append log for connection.
            if anta_inf == 1 or anta_inf == 2 : self.Log.PushConnected('Antenna-A')
            else : self.Log.PushDisconnected('Antenna-A')
             
        if self.AntCommB.IsConnect() == 1 : antb_inf = 1
        elif self.McuComm.SubAnt.IsConnect() == 1 : antb_inf = 2
        if self.AntBInterface != antb_inf :
            self.AntBInterface = antb_inf
            #Set antenna information.
            if antb_inf == 1 : self.AntB = self.AntCommB.Ant
            elif antb_inf == 2 : self.AntB = self.McuComm.Ant
            else : self.AntB = None
            #Append log for connection.
            if antb_inf == 1 or antb_inf == 2 : self.Log.PushConnected('Antenna-B')
            else : self.Log.PushDisconnected('Antenna-B')
            
        # ** old code    
        # if self.AntCommA.IsConnect() == 1 : 
            # self.AntAInterface = 1
            # self.AntA = self.AntCommA.Ant
        # elif self.AntSubComm.IsConnect() == 1 : 
            # self.AntAInterface = 2
            # self.AntA = self.AntSubComm.Ant
        # else : 
            # self.AntAInterface = 0
            # self.AntA = None
            
        # if self.AntCommB.IsConnect() == 1 : 
            # self.AntBInterface = 1
            # self.AntB = self.AntCommB.Ant
        # elif self.McuComm.SubAnt.IsConnect() == 1 : 
            # self.AntBInterface = 2
            # self.AntB = self.McuComm.Ant
        # else : 
            # self.AntBInterface = 0
            # self.AntB = None
    def GetJson(self,token):
        dat = { 
            'Id':token,
            'OpTime':self.Vcs.OpTime,
            'Interface':{},
            'Mcu':{},
            'Gps':{},
            'Gyro':{},
            'BlockControl':{},
            'Vcs':{},
            'Device':{},
            'Modem':{
                'Connection':self.AbsSer.CheckConnection(),
                'Class':self.Sys.GetAbsClass(),
                'Message':{},
            },
			'Event':pj.Log.Get(),
            'GyroSyncStepStartupFlag':self.GyroSyncStepStartupFlag
        } 
        dat['Interface'] = self.GetInterfaceStatusArray()
        dat['Mcu'] = self.McuComm.Mcu.GetArray()
        dat['Gps'] = self.GpsMgr.GetArray()
        dat['Gyro'] = self.GyroMgr.GetArray()
        dat['BlockControl'] = self.BlockCtl.GetArray()
        dat['Vcs'] = self.GetVcs()
        dat['Device'] = self.GetDevice()
        dat['Modem']['Message'] = self.AbsComm.Sat.GetArray()
        return json.dumps(dat)
        
    def GetDevice(self):
        dat = { 
            'Temperature':Util.GetTemperature(),
            'Version':self.Version,
        }
        return dat
    def GetVcs(self):
        dat = { 
            'Ant':{
                'A': self.AntATrackOn,
                'B': self.AntBTrackOn,
                'Auto': (1 if self.Vcs.AntSel == 3 else 0),
                'Link': self.Vcs.Link,
                'Lock': self.Vcs.Lock,
                'LockFromConsole': self.Vcs.LockFromConsole,
                'Sel': self.Vcs.AntSel,
                'AutoA': (1 if self.AntAutoSel == 1 else 0),
                'AutoB': (1 if self.AntAutoSel == 2 else 0),
                'SourceA': self.AntAInterface,
                'SourceB': self.AntBInterface,
                'IfSel': self.AntIfSel,
            },
			'RefFreq':{
                'Sel': self.Vcs.RefFreqSel,
            }
        }
        return dat
    def GetInterfaceStatusArray(self):
        dat = { 
            'AntCommA':{
                'Connection': self.AntCommA.IsConnect(),
                'Ant': self.AntCommA.Ant.RcvCnt,
                'AntNmea': self.AntCommA.Ant.Nmea.GetValidArray(),
            },
            'AntCommB':{
                'Connection': self.AntCommB.IsConnect(),
                'Ant': self.AntCommB.Ant.RcvCnt,
                'AntNmea': self.AntCommB.Ant.Nmea.GetValidArray(),
            },
            'AntSub':{
                'Connection': self.AntSubComm.IsConnect(),
                'Ant': self.AntSubComm.Ant.RcvCnt,
                'AntNmea': self.AntSubComm.Ant.Nmea.GetValidArray(),
            },
            'McuComm':{
                'Connection': self.McuComm.IsConnect(),
                'AntConnection': self.McuComm.SubAnt.IsConnect(),
                'Ant': self.McuComm.Ant.RcvCnt,
                'AntNmea': self.McuComm.Ant.Nmea.GetValidArray(),
                'Mcu': self.McuComm.Mcu.RcvCnt,
            },
            'GyroComm':{
                'Connection': self.GyroComm.IsConnect(),
                'IsRcvNmea': self.GyroComm.SubExtNmea.IsConnect(),
                'Nmea': self.GyroComm.Nmea.GetValidArray(),
            },
            'NmeaComm':{
                'Connection': self.NmeaComm.IsConnect(),
                'IsRcvNmea': self.NmeaComm.SubExtNmea.IsConnect(),
                'Nmea': self.NmeaComm.Nmea.GetValidArray(),
            },
            'ConComm':{
                'Connection': self.ConComm.IsConnect(),
                'IsRcvNmea': self.ConComm.SubExtNmea.IsConnect(),
                'Nmea': self.ConComm.Nmea.GetValidArray(),
            },
            'WebComm':{
                'Request': self.WebComm.RcvCnt,
            },
            'AbsComm':{
                'Connection': self.AbsComm.IsConnect(),
            },
        }
        return dat


pj = Project()
        
        
